/*
 * Program MMAMWM V0.99.9 or higher
 * (C) by Benjamin Drieu (e-mail: drieu@bocal.cs.univ-paris8.fr)
 * Feb 24rd 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Publis License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "main.h"



/* Cette fonction est celle qui va gerer le deplacement vertical de la
   fenetre vers le haut */
void
myRaise (Window fenetre)
{
  XRaiseWindow (display, fenetre);
}



/* Cette fonction deplace la fenetre vers le bas */
void
myLower (Window fenetre)
{
  XLowerWindow (display, fenetre);
}



/* Cette fenetre gere le deplacement de la fenetre sur l'ecran.
 Lorsque le bouton de la souris est presse sur la barre, et que le
 pointeur est deplace, alors cette procedure est appelee */
void
myMove (Window fenetre)
{
  XEvent event, event2;
  Window root, child;
  XWindowAttributes wattr;
  int move_x, move_y, win_x, win_y;
  int toto;
  int mask;

  /* Les attributs de la fenetre vont nous servir par la suite (taille
     et placement) */
  XGetWindowAttributes (display, fenetre, &wattr);

  /* Seule cette fenetre recevra les evenements souris ButtonRelease,
     PointerMotion, et ButtonPressMask */
  XGrabPointer (display, fenetre, True, ButtonReleaseMask |
		PointerMotionMask | ButtonPressMask,  GrabModeAsync,
		GrabModeAsync, None, 0, CurrentTime);

  /* On recupere les coordonnees de la souris */
  XQueryPointer (display, fenetre, &root,
		 &child, &move_x, &move_y, &win_x, &win_y, &mask);
  
  /* La fonction drawRectangle dessin un joli rectangle avec les deux
     diagonales des dessinees */
  drawRectangle (move_x-win_x+1, move_y-win_y+1, 
		 wattr.width, wattr.height);

  /* Il ne reste plus qu'a boucler tant qu'on ne rencontre pas un
     evenement ButtonRelease */
  while (1)
    {
      XNextEvent (display, &event);

      /* Petite bidouille pour ne pas recuperer tous les evenements
         PointerMotion (ce qui est penible sur les machines a faible
         vitesse d'affichage */
      while(XEventsQueued(display, QueuedAfterReading) > 0)
	{
	  XPeekEvent(display, &event2);
	  if(event2.type != MotionNotify) 
	    break;
	  XNextEvent(display, &event);
	}

      /* Si le bouton a ete relache, alors on s'arrete */
      if (event.type == ButtonRelease)
	{
	  drawRectangle (move_x-win_x+1, move_y-win_y+1, 
			 wattr.width, wattr.height);
	  break;
	}

      /* On dessine a chaque fois un rectangle a la position du
         pointeur */
      drawRectangle (move_x-win_x+1, move_y-win_y+1, 
		     wattr.width, wattr.height);
      XQueryPointer (display, fenetre, &root, &child, &move_x,
		     &move_y, &toto, &toto, &mask);  
      drawRectangle (move_x-win_x+1, move_y-win_y+1, 
		     wattr.width, wattr.height);
    }
  /* On deplace la fenetre a la position du pointeur */
  XQueryPointer (display, fenetre, &root, &child, &move_x, &move_y,
 		 &toto, &toto, &mask); 
  XMoveWindow (display, fenetre, move_x-win_x-1, move_y-win_y-1);

  /* On en profite pour la mettre tout en haut de la pile */
  myRaise (fenetre);

  /* Il ne reste plus qu'a liberer le pointeur */
  XUngrabPointer (display, CurrentTime);
  XFlush (display);
}




/* Cette fonction est celle qui gere la redimension des fenetres.
   Elle ressemble beaucoup a la fonction myMove */
void
myResize (WindowListe * tmp)
{
  XEvent event, event2;
  Window root, child;
  XWindowAttributes wattr, wattr_client;
  int move_x, move_y, win_x, win_y;
  int mask;

  /* Des informations sur les fenetres nous seront utiles */
  XGetWindowAttributes (display, tmp->window, &wattr);
  XGetWindowAttributes (display, tmp->client, &wattr_client);
  
  /* Les evenements seronts limites a la fenetre */
  XGrabPointer (display, tmp->window, True, ButtonReleaseMask |
		PointerMotionMask | ButtonPressMask,  GrabModeAsync,
		GrabModeAsync, None, 0, CurrentTime);

  /* Nous changeons le curseur lors de la manipulation de la fenetre */
  XDefineCursor (display, RootWindow(display, screen_num), resize_cursor);
  XDefineCursor (display, tmp->client, resize_cursor);

  /* La souris est deplacee sur le coin bas droit de la fenetre */
  XWarpPointer (display, RootWindow(display,screen_num), tmp->window, 
		0, 0, 0, 0, wattr.width, wattr.height); 

  /* On recupere les coordonees de la souris */
  XQueryPointer (display, RootWindow (display, screen_num), &root,
		 &child, &move_x, &move_y, &win_x, &win_y, &mask);

  /* Tant qu'on n'a pas recu d'evenement MouseEvent, on boucle */
  while (1)
    {

      /* ON en profite pour dessiner un joli rectangle a la place de
         la fenetre */
      drawRectangle (wattr.x, wattr.y, 
		     move_x-wattr.x, move_y-wattr.y); 

      XNextEvent (display, &event);

      /* On n'ecoute que certains evenements */
      while(XEventsQueued(display, QueuedAfterReading) > 0) 
 	{ 
 	  XPeekEvent(display, &event2); 
 	  if(event2.type != MotionNotify)  
	    {
	      break; 
	    }
 	  XNextEvent(display, &event); 
 	} 

      /* Si c'est un evenement ButtonRelease, on quitte la boucle */
      if (event.type == ButtonRelease)
	{
	  break;
	}

      /* On dessine des rectangles pour suivre le redimensionnement de
         la fenetre */
      drawRectangle (wattr.x, wattr.y, 
		     move_x-wattr.x, move_y-wattr.y); 
      XQueryPointer (display, RootWindow (display, screen_num), &root,
		     &child, &move_x, &move_y, &win_x, &win_y, &mask);
    }

  /* Encore un rectangle pour eviter des marques persistantes a
     l'ecran */
  XQueryPointer (display, RootWindow (display, screen_num), &root,
		 &child, &move_x, &move_y, &win_x, &win_y, &mask);
  drawRectangle (wattr.x, wattr.y, 
		 move_x-wattr.x, move_y-wattr.y); 

  /* On teste si la taille de la fenetre est negative en hauteur ou en
     largeur.  Si c'est le cas, alors la fenetre aura la valeur
     absolue comme taille, et sera deplacee en haut ou a gauche */
  if (move_x-wattr.x<0)
    {
      int toto=move_x;
      XMoveWindow (display, tmp->window, move_x, wattr.y); 
      move_x=wattr.x;
      wattr.x=toto;
    }
  if (move_y-wattr.y<0)
    {
      int toto=move_y;
      XMoveWindow (display, tmp->window, wattr.x, move_y); 
      move_y=wattr.y;
      wattr.y=toto;
    }

  /* Finalement, on deplace la fenetre */
  XResizeWindow (display, tmp->window, move_x-wattr.x-1,
		 move_y-wattr.y-1); 
  XResizeWindow (display, tmp->client,
		 move_x-wattr.x-BORDER_WIDTH-MARGIN_WIDTH-
		 2*wattr_client.border_width-1,
		 move_y-wattr.y-(2*BORDER_WIDTH)-
		 2*wattr_client.border_width-1);

  /* Tant qu'a faire, on la mete tout en haut de la pile */
  myRaise (tmp->window);

  /* On libere la souris */
  XUngrabPointer (display, CurrentTime);

  /* On remet le curseur standard */
  XUndefineCursor (display, tmp->client);
  XDefineCursor (display, RootWindow(display, screen_num),
		 normal_cursor); 

  /* Et on remete le display en place */
  XFlush (display);
}



/* Cette fonction iconifie la fenetre, et met l'attribut de la fenetre */
void
myIconify (WindowListe *tmp)
{
  tmp->iconified=True;
  XUnmapWindow (display, tmp->window);
}



