/*
 * Program MMAMWM V0.99.9 or higher
 * (C) by Benjamin Drieu (e-mail: drieu@bocal.cs.univ-paris8.fr)
 * Feb 24rd 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Publis License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "main.h"
#include <X11/xpm.h>

int
main (int argc, char **argv)
{
  XEvent event;
  WindowListe * tmp;

  /* Creation de la liste chainees des clients */
  initListe ();

  /* Initialisations X */
  initX ();

  /* Cree la liste des clients deja existants */
  getAllWindows ();

  /* Boucle principale */
  while (1)
    {
      /* On attend l'evenement suivant */
      XNextEvent (display, &event);
      
      /* Y'a plus qu'a le gerer selon son type */
      switch (event.type)
	{
	case MotionNotify:
	  /* Le pointeur a bouge */
	  /* Ce qui nous interresse est le deplacement du curseur sur
	     la RootWindow */
	  if (event.xmotion.window == 
	      RootWindow(display, screen_num)) 
	    {
	      /* Rien pour l'instant, mais peut-etre des icones sur le
		 RootWindow (ca ne sert a rien, mais c'est un concept
		 a essayer) */
	    }
	  break;

	case Expose:
	  /* Une fenetre doit etre redessinee */
	  tmp=dansLaListe (event.xexpose.window);
	  if (tmp)
	    {
	      /* C'est une des barres... */
	      redrawBarre (tmp);
	    }
	  /* Sinon... rien */
	  break;

	case EnterNotify:
	  /* Le pointeur entre dans une des fenetres */
	  if (isClic (event.xcrossing.window))
	    {
	      enterBarre (event.xcrossing.window);
	    }
	  else
	    {
	      tmp=dansLaListeClient (event.xcrossing.window);
	      if (tmp != NULL && tmp->cmap != (Colormap) NULL)
		installCmap (tmp->cmap);
	      else
		installCmap (XDefaultColormap(display, screen_num));
	    }
	  break;

	case LeaveNotify:
	  /* Le pointeur vient de sortir de la fenetre */
	  leaveBarre (event.xcrossing.window);
	  break;

	case ButtonPress:
	  /* On clique sur une fenetre - C'est la partie la plus
	     complexe */
	  tmp=isClic (event.xbutton.window);
	  if (tmp)
	    {
	      myIconify (tmp);
	    }
	  else if (dansLaListe (event.xbutton.window))
	    {
		
	      /* C'est une des barres... */
	      /* On va donc regarder quel bouton a ete clique */
	      switch (event.xbutton.button)
		{
		case Button1:
		  /* Bouton 1: deplacement + montee de la fenetre (la
		     deplacement la met au premier plan de toute
		     facon, meme pour un deplacement nul) */
		  myMove (event.xbutton.window); 
		  break;
		  
		case Button2:
		  /* On met la fenetre a l'arriere plan */
		  myLower (event.xbutton.window); 
		  break;
		  
		case Button3:
		  /* Pour l'instant, rien (mais certainement un
		     deplacement d'une fenetre a l'autre) */
		  tmp=dansLaListeClient (event.xbutton.window);
		  myResize (tmp);
		  break;
		}
	    }
	  /* L'evenement est sur le fond de l'ecran */
	  else if (event.xbutton.window == RootWindow(display,screen_num))
	    {
	      switch (event.xbutton.button)
		{
		case Button1:
		  /* On Execute une xterm */
		  if (!fork()) 
		    { 
		      execl ("/usr/X11R6/bin/xterm", "xterm", NULL);
		    } 
		  break;

		case Button2:
		  /* On va afficher le menu des fenetres iconifiees */
		  updateMenu();
		  
		  /* Si le menu est vide, pas besoin de l'afficher */
		  if (numMenuEntries > 0)
		    popupMenu(); 
		  break;
		}
	    }
	  /* Alors, on sort */
	  break;

	case CreateNotify:
	  /* Une fenetre est cree.  Si elle n'est pas deja dans la
             liste, alors on la manage */
	  if (! dansLaListeClient (event.xcreatewindow.window))
	    {
	      manageWindow (event.xcreatewindow.window);
	    }
	  break;

	case DestroyNotify:
	  /* C'est l'inverse: la fenetre est detruite */
	  if (dansLaListeClient (event.xcreatewindow.window))
	    {
	      /* On detruit physiquement la fenetre qui recoit
                 l'evenement (donc en theorie la fentre "pere", donc
                 la fenetre de decoration */
	      XDestroyWindow (display, event.xdestroywindow.event);
	      /* Et on la supprime de la liste des fenetres managees */
	      enleverListe (event.xdestroywindow.event);
	    }
	  break;

	case UnmapNotify:
	  /* La fenetre est "unmapee", c'est a dire iconifiee.  Si
             elle est dans la liste des fenetres managees, alors on
             l'iconifie reelement (on la place dans le menu des
             fenetres iconifiees) */
	  tmp=dansLaListeClient(event.xunmap.event);
	  if (tmp)
	    {
	      myIconify (tmp);
	    }
	  break;
	  
	case MapNotify:
	  /* C'est l'inverse, on desiconifie une fenetre (il est
             possible qu'un bug mineur apparaisse lorsque le client a
             mappe la fenetre lui meme */
	  tmp=dansLaListeClient(event.xunmap.event);
	  if (tmp)
	    {
	      XMapWindow (display, tmp->window);
	    }
	  break;

	case ResizeRequest:
	  /* La fenetre s'est redimensionnee toute seule, alors on
             redimenssionne le bord avec elle */
	  tmp=dansLaListeClient (event.xresizerequest.window);
	  if (tmp)
	    {
	      XResizeWindow (display, tmp->window, 
			     event.xresizerequest.width+
			     BORDER_WIDTH+MARGIN_WIDTH+3,
			     event.xresizerequest.height+
			     (2*BORDER_WIDTH)+3);
	      XResizeWindow (display, tmp->client, 
			     event.xresizerequest.width,
			     event.xresizerequest.height);
	    }
	  break;

	} /* switch */
    } /* while */
} /* main */


