/*
 * Program MMAMWM V0.99.9 or higher
 * (C) by Benjamin Drieu (e-mail: drieu@bocal.cs.univ-paris8.fr)
 * Feb 24rd 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Publis License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "main.h"


/* Cette fonction initialise le menu en creant la fenetre de menu */
void
createMenu ()
{
  menuWin=XCreateSimpleWindow (display, 
			       RootWindow(display, screen_num), 
			       10, 10, 100, MENU_HEIGHT, 1, 
			       BlackPixel(display, screen_num),
			       PeachPuff.pixel);
  XSelectInput (display, menuWin, PointerMotionMask|ButtonReleaseMask); 
}



/* Cette fonction met a jour le menu:
    - Elle detruit les sous-fenetres deja existantes 
    - Elle compte le nombre de fenetres deja iconifiees
    - Elle cree les sous-fenetres destinees a contenir le 
      nom de chacune des fenetres iconifiees */
void
updateMenu ()
{
  WindowListe *tmp=debut;
  
  XDestroySubwindows (display, menuWin);

  if (debut == NULL)
    return;

  numMenuEntries=0;

  do
    {
      if (tmp->iconified == True)
	{
	  menuEntries[numMenuEntries++]=tmp;
	}
      tmp=tmp->s;
    }
  while (tmp != debut);

  if (numMenuEntries != 0)
    makeMenu ();

}



/* Cette fonction cree chacune des sous-fenetres du menu : chaque
   ligne du menu est une sous-fenetre qui gere elle-meme les
   evenements qui lui arrivent.  C'est beacoup plus simple que de
   faire une fenetre unique et de calculer la ligne sur laquelle on se
   trouve grace a un offset */
void
makeMenu ()
{
  int i;
  
  largeur_max=getMax();
  XResizeWindow (display, menuWin, largeur_max, MENU_HEIGHT*numMenuEntries);

  for (i=0; i<numMenuEntries; i++)
    {
      menuLines[i]=
	XCreateSimpleWindow (display, menuWin, 
			     0, i*MENU_HEIGHT, largeur_max, MENU_HEIGHT, 0,
			     BlackPixel(display, screen_num),
			     PeachPuff.pixel);
      XSelectInput (display, menuLines[i],
		    ButtonReleaseMask|EnterWindowMask|LeaveWindowMask); 
    }
}



/* Cette fonction est celle qui va mapper le menu, mais aussi qui va
   dessiner toutes les lignes du menu.  Elle va de plus attendre que
   l'utilisateur ai relache le bouton de la souris pour savoir quelle
   sous-fenetre a ete selectionnee */
void
popupMenu ()
{
  XEvent event;
  Window root, child;
  int win_x, win_y;
  int mask, i;

  /* On affiche le menu */
  XQueryPointer (display, RootWindow(display, screen_num), &root,
		 &child, &win_x, &win_y, &win_x, &win_y, &mask);
  XMoveWindow (display, menuWin, win_x-8, win_y-MENU_HEIGHT/2);
  XRaiseWindow (display, menuWin);
  XMapWindow (display, menuWin);
  XGrabPointer (display, menuWin, True, ButtonReleaseMask |
		PointerMotionMask | ButtonPressMask,  GrabModeAsync,
		GrabModeAsync, None, 0, CurrentTime);

  /* On mappe toutes les fenetres */
  for (i=0; i<numMenuEntries; i++)
    {
      XMapWindow (display, menuLines[i]);
      drawMenuLine (i, False);
    }

  /* On attend l'evenement ButtonRelease */
  while (1)
    {
      XNextEvent (display, &event);
      if (event.type == ButtonRelease)
	{
	  break;
	}
      
      /* Au besoin, on redessine une ligne afin de donner un look
         different a la ligne selectionnee */
      for (i=0; i<numMenuEntries; i++)
	{
	  if (menuLines[i] == event.xcrossing.window &&
	      event.type == EnterNotify)
	    {
	      drawMenuLine (i, True);
	    }
	  else if (menuLines[i] == event.xcrossing.window &&
		   event.type == LeaveNotify)
	    {
	      drawMenuLine (i, False);
	    }
	}
    }

  /* On mappe la fenetre dont l'indice est le meme que la ligne
     selectionnee */
  for (i=0; i<numMenuEntries; i++)
    {
      if (menuLines[i] == event.xbutton.window)
	{
	  menuEntries[i]->iconified=False;
	  XMapWindow (display, menuEntries[i]->window);
	}
    } 

  /* On relache le pointeur et on en profite pour raffraichir le
     display */
  XUngrabPointer (display, CurrentTime);
  XFlush (display);
  XUnmapWindow (display, menuWin);
}



/* Cette fonction est celle qui redissine une des lignes du menu.
   L'argument "selected" permet de savoir si on la dessine avec un
   look normal, ou si au contraire la ligne est selectionnee, et
   necessite le dessin d'une bordure de look 3d */
void
drawMenuLine (int pos, Bool selected)
{
  /* On efface la feneter et on dessine une chaine de caracteres par
     dessus */
  XClearWindow (display, menuLines[pos]);
  XDrawString (display, menuLines[pos], gc_fonce, 5,
	       MENU_HEIGHT-(hauteur_max/2), 
	       menuEntries[pos]->name,
	       strlen(menuEntries[pos]->name));   

  /* Dessin d'un cadre 3d */ 
  if (selected)
    {
      XSetLineAttributes (display, gc_fonce, 2, 
			  LineSolid, CapButt, JoinBevel);   
      XDrawRectangle (display, menuLines[pos], gc_fonce, 
		      1, 1, largeur_max-1, MENU_HEIGHT-2); 
      XSetLineAttributes (display, gc_clair, 2, 
			  LineSolid, CapButt, JoinBevel);   
      XDrawLine (display, menuLines[pos], gc_clair, 
		 0, 1, largeur_max-1, 1); 
      XDrawLine (display, menuLines[pos], gc_clair, 
		 largeur_max-1, 0, largeur_max-1, MENU_HEIGHT-1); 
      XDrawPoint (display, menuLines[pos], gc_fonce, 0, 1);  
      XDrawPoint (display, menuLines[pos], gc_fonce, 
		  largeur_max-2, MENU_HEIGHT-2);  
    }
}



/* Fonction utile qui renvoie la taille maximum en largeur d'une
   chaine de caracteres donnee, avec comme police la police du menu */
int
getMax ()
{
  int i, maximum=0, tmp=0;
  XFontStruct *fstruct;
  XCharStruct tmpCharS;
  int direction, ascent, descent;

  /* On charge la police */
  fstruct=XQueryFont(display, police);

  /* Pour calculer la hauteur du texte, on peut utiliser n'importe
     quelle chaine */
  XTextExtents (fstruct, "N'importe quoi !!!", 18,
		&direction, &ascent, &descent, &tmpCharS);
  hauteur_max=ascent+descent;

  /* On calcule par contre la largeur maximum dans toutes les chaines
     contenues dans le menu */
  for (i=0; i<numMenuEntries; i++)
    {
      tmp=XTextWidth (fstruct, menuEntries[i]->name,
		      strlen(menuEntries[i]->name));
      if (tmp > maximum)
	{
	  maximum=tmp;
	}
    }
  return max(maximum,100)+10;
}



