/*
 * Program MMAMWM V0.99.9 or higher
 * (C) by Benjamin Drieu (e-mail: drieu@bocal.cs.univ-paris8.fr)
 * Feb 24rd 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Publis License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "main.h"


/* Cette fonction est lancee au demarrage.  Elle recupere toutes les
   fenetres de niveau superieur du display, et les manage si'il le
   faut */
void
getAllWindows ()
{
  Window root_return, parent_return, *children_return;
  unsigned int nchildren_return;
  int i;

  XQueryTree (display, RootWindow(display, screen_num),
	      &root_return, &parent_return, &children_return,
	      &nchildren_return);
  for (i=0; i<nchildren_return; i++)
    { 
      manageWindow (children_return[i]);
    }
}



/* On manage s'il le faut une fenetre particuliere */
void
manageWindow (Window fenetre)
{
  Window bord, clic;
  XWindowAttributes xswa;
  XGetWindowAttributes(display, fenetre, &xswa);

  /* On determine s'il faut manager la fenetre si elle est visible, et
     si ce n'est pas le menu */
  if(!xswa.override_redirect && fenetre != menuWin)
    {  
      /* On cree la fenetre de bord */
      bord=XCreateSimpleWindow 
	(display, RootWindow(display, screen_num), 
	 xswa.x-MARGIN_WIDTH-xswa.border_width-1,
	 xswa.y-BORDER_WIDTH-xswa.border_width-1, 
	 xswa.width+MARGIN_WIDTH+BORDER_WIDTH+2*xswa.border_width,
	 xswa.height+BORDER_WIDTH*2+2*xswa.border_width, 1,  
	 BlackPixel(display, screen_num), 
	 PeachPuff.pixel);

      /* Ainsi que la fenetre d'iconification */
      clic=XCreateSimpleWindow 
	(display, bord,
	 5, 5, 8, 8, 1,
	 BlackPixel(display, screen_num), 
	 WhitePixel(display, screen_num));
      /* Le curseur de cette derniere sera particulier */
      XDefineCursor (display, clic, iconify_cursor);

      /* On verifie que la colormap est propre a la fenetre */
      if (xswa.colormap != XDefaultColormap(display, screen_num))
	{
	   addToListe (bord, fenetre, clic, getName(fenetre), xswa.colormap);
	} 
      else 
	{
	  addToListe (bord, fenetre, clic, getName(fenetre), 
		      (Colormap) NULL);
	}

      /* On rend la fenetre de bord pere de la fenetre cliente */
      XReparentWindow (display, fenetre, bord,
		       MARGIN_WIDTH, 
		       BORDER_WIDTH);

      /* On deplace la fenetre client au bon endroit dans la fentre
         bord */
      XMoveWindow (display, fenetre, MARGIN_WIDTH, BORDER_WIDTH); 

      /* MAGIQUE: quand le window manager sera fini, toutes les
         fenetres du save-set seront automariquement reparentees a la
         bonne fenetre */
      XAddToSaveSet (display, fenetre);

      /* On n'a plus qu'a dire quels evenements gerer */
      XSelectInput (display, bord, ButtonPressMask | ExposureMask |
		    SubstructureNotifyMask | PointerMotionMask |
		    EnterWindowMask);  
      XSelectInput (display, clic, ButtonPressMask | EnterWindowMask |
		    LeaveWindowMask | PointerMotionMask); 
      XSelectInput (display, fenetre, ResizeRedirectMask); 

      /* ET on mappe le tout ! */
      XMapWindow(display, bord);
      XMapWindow(display, clic);
      XMapWindow(display, fenetre);
    }  
}



/* Cette fonction dessine un rectangle sur la RootWindow avec les deux
   diagonales (plus pratique) */
void
drawRectangle (int x, int y, int w, int h)
{
  
  if (w<0)
    {
      x=x+w;
      w=-w;
    }
  if (h<0)
    {
      y=y+h;
      h=-h;
    }
  XDrawRectangle (display, RootWindow(display, screen_num),
		  gc_xor, x, y, w, h);
  XDrawLine (display, RootWindow(display, screen_num),
	     gc_xor, x, y, x+w, y+h);
  XDrawLine (display, RootWindow(display, screen_num),
	     gc_xor, x+w, y, x, y+h);
}



/* Cette fonction purement utilitaire recupere le nom de la fenetre et
   le retourne sous forme de chaine de caracteres */
char *
getName (Window win)
{
  XTextProperty textp;
  XGetWMName (display, win, &textp);
  return textp.value;
}



/* Cette fonction installe la colormap passee en argument.  Elle est
   typiquement appelee lorsque le pointeur de la souris entre dans une
   fenetre dont la colormap est propre */
void
installCmap (Colormap cmap)
{
  XInstallColormap (display, cmap);
}



