/*
 * Program MMAMWM V0.99.9 or higher
 * (C) by Benjamin Drieu (e-mail: drieu@bocal.cs.univ-paris8.fr)
 * Feb 24rd 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Publis License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "main.h"
#include <X11/xpm.h>


void
initX ()
{
  /* Ouverture du display */
  openDisplay ();
  screen_num=DefaultScreen(display);

  /* Creation des couleurs PeachPuff */
  initXColors ();

  /* Creation des GC */
  initGC ();

  /* Initialisations diverses */
  initXDivers ();

}



/* On ouvre simplement le display */
void
openDisplay ()
{
  display=XOpenDisplay (NULL);
  if (display == NULL)
    {
      fprintf (stderr, "Ouverture du display %s impossible: YUCK!\n",
	       XDisplayName(NULL));
      exit (-1);
    }
}



/* On initialise les GCs:
   - gc_noir: un gc tout simple qui ecrit en noir 
   - gc_blanc: idem, mais en blanc
   - gc_clair: un gc qui dessine en marron clair (PeachPuff)
   - gc_fonce: un gc qui dessine en marron fonce
   - gc_fond: un gc qui dessine en marron moyen
   - gc_xor: un gc etrange qui dessine avec la fonction xor, et qui
   dessine aussi par dessus les sous-fenetres */
void
initGC ()
{
  gc_noir=XCreateGC (display, RootWindow(display, screen_num), 0, NULL);
  XSetForeground (display, gc_noir, BlackPixel(display, screen_num));
  police=XLoadFont(display, FONT_NAME);
  police2=XLoadFont(display, FONT_NAME_2);
  if (! police || ! police2)
    {
      fprintf (stderr, "Impossible de charger la police\n");
    }
  
  gc_blanc=XCreateGC (display, RootWindow(display, screen_num), 0, NULL);
  XSetForeground (display, gc_blanc, WhitePixel(display, screen_num));

  gc_clair=XCreateGC (display, RootWindow(display, screen_num), 0, NULL);
  XSetForeground (display, gc_clair, PeachPuffClair.pixel);

  gc_fonce=XCreateGC (display, RootWindow(display, screen_num), 0, NULL);
  XSetForeground (display, gc_fonce, PeachPuffFonce.pixel);
  XSetFont (display, gc_fonce, police);

  gc_barre=XCreateGC (display, RootWindow(display, screen_num), 0, NULL);
  XSetForeground (display, gc_barre, PeachPuffFonce.pixel);
  XSetFont (display, gc_barre, police2);

  gc_fond=XCreateGC (display, RootWindow(display, screen_num), 0, NULL);
  XSetForeground (display, gc_fond, PeachPuff.pixel);

  gc_xor=XCreateGC (display, RootWindow(display, screen_num), 0, NULL);
  XSetSubwindowMode (display, gc_xor, IncludeInferiors);
  /* Alors la, je ne sais plus pourquoi ca marche !!! */
  XSetForeground (display, gc_xor, BlackPixel(display, screen_num) 
		  ^ WhitePixel(display, screen_num));
  XSetFunction (display, gc_xor, GXxor); 
}




/* Initialisations diverses:
   - Les evenements geres par la RootWindow 
   - Fermeture des connections sockets en cours (il parait que sinon ca ne marche pas)
   - Creation des curseurs
   - Creation du menu */
void
initXDivers ()
{
  XSelectInput (display, RootWindow(display, screen_num),
		ButtonPressMask | PointerMotionMask | 
		SubstructureNotifyMask | ExposureMask);  
						    
  if (fcntl (ConnectionNumber(display), F_SETFD, 1) == -1)
    {
      fprintf (stderr, "Gaffe au fd TCP !!!\n");
    }

  iconify_cursor=XCreateFontCursor (display, XC_based_arrow_down);
  normal_cursor=XCreateFontCursor (display, XC_left_ptr);
  resize_cursor=XCreateFontCursor (display, XC_plus);
  XDefineCursor (display, RootWindow(display, screen_num), normal_cursor);
  
  createMenu(); 
}  



/* Initialisation de trois couleurs: fonce, normal, clair */
void
initXColors ()
{
  XAllocNamedColor (display, 
		    XDefaultColormap (display, screen_num),
		    COLOR_FONCE, &PeachPuffFonce, &PeachPuffFonce);  

  XAllocNamedColor (display, 
		    XDefaultColormap (display, screen_num),
		    COLOR_NORMAL, &PeachPuff, &PeachPuff);

  XAllocNamedColor (display, 
		    XDefaultColormap (display, screen_num),
		    COLOR_CLAIR, &PeachPuffClair, &PeachPuffClair);  
}



